#!/usr/bin/env python
# coding: utf-8

"""
    quickCrypt.py
    
    A small program to quickly and easily encrypt small files 
    using the Phelix algorithm.  ........
    
    The key may be any string of up to 32 characters.  Be sure 
    to enclose the key in quotes if it has imbedded spaces.  
    Because the key is typed, some may consider the key weak.  
    Stronger keys are possible if you modify this program.  If
    the key is not on the commandline, the user will be asked 
    to type it in (no echo).
    
    The new encrypted file is named the same as the original 
    file but with a '.phx' suffix added.  The decrypted file 
    is written without the '.phx' suffix.  If the output file 
    already exists and is about to be overwritten, the user is
    asked for permission.
    
    quickCrypt does not chunk large files so file size is 
    limited by available memory.  Sizes is the many-megabyte 
    range should work fine.  A minor rewrite would be required 
    to process files of unlimited sizes.  
    
    On my 1.5 GHz PPC G4 miniMac with 1 GB RAM and 7200 RPM 
    drive, encrypting and writing a 10 MB file took well under 
    a second.  A 118 MB file took about 7.5 seconds.  About 1/2 
    that time was I/O.  Excluding I/O and other setup, Phelix is 
    encrypting and hashing about 36 MB per second.  Decryption 
    is a little slower at about 31 MB per second.  All this is 
    with the "unoptimized" C version of Phelix.  By contrast,
      Twofish   21.3 MB per sec
      AES       19.2 MB per sec
      blowfish  19.1 MB per sec
      cast5     17.6 MB per sec
      anubis    15.1 MB per sec
      xtea      13.8 MB per sec
      3des       7.6 MB per sec
    
    Usage:
        python quickCrypt.py <filename> [<key>]
    
    
    Larry Bugbee
    March 2007
    
"""

import pyPhelix, sys, md5, time, os, getpass, random, struct

#---------------------------------------------------------------
# constants

suffix = '.phx'

#---------------------------------------------------------------
# utilities

def okToProceed(filename):
    if os.path.exists(filename):
        prompt = '  "%s" exists.  overwrite? [y/n]: ' % filename
        response = raw_input(prompt)
        if not (response and response.lower()[0] == 'y'):
            print '  aborting: original file preserved'
            sys.exit()
    
def savetofile(filename, content):
    f = open(filename, 'wb')
    f.write(content)
    f.close()
    
def loadfmfile(filename):
    f = open(filename, 'rb')
    content = f.read()
    f.close()
    return content

def byt2hex(bs):
    h = ''
    for c in bs:
        h = h + ('0'+hex(ord(c))[2:])[-2:]
    return h

# a quick way to get exactly 128 bits
def quick128(data):
    md = md5.md5()
    md.update(data)
    return md.digest()

def makeNounce():
    # sufficient time delay on most processors to ensure a 
    # unique nonce, but adding a supposedly random number can't
    # hurt
    time.sleep(0.1)
    nonce  = struct.pack('d', time.time())
    nonce += struct.pack('d', random.getrandbits(64))

    # probably unnecessary, but here are a few more identifiers 
    # that might further reduce the probability of time being 
    # reused.  
    #   ssid = os.environ.get('SECURITYSESSIONID', 'na')
    #   nonce += ' %d %d %d %s' % (os.getpid(), 
    #                              os.getsid(os.getpid()), 
    #                              os.getuid(),
    #                              ssid,
    #                             )
    return nonce    

def usage():
    print '\n  usage:\n    python %s <filename> <key>\n' % sys.argv[0]
    sys.exit()

#---------------------------------------------------------------

def main():
    if len(sys.argv) > 1:
        filename = sys.argv[1]
    if len(sys.argv) == 3:
        key = sys.argv[2]
    elif len(sys.argv) == 2:
        key = getpass.getpass('  enter your key: ')
    else:
        usage()
    
    if len(key) > 32:
        print '\n  *** key must be 32 characters or less ***\n'
        sys.exit()
        
    content = loadfmfile(filename)    
    encrypt = not filename.endswith(suffix)
    
    if encrypt:
        newfilename = filename+suffix
        okToProceed(newfilename)
        nonce = makeNounce()
        IV    = quick128(nonce)  # to ensure IV is exactly 128 bits
        time.clock()
        ciphertext, mac = pyPhelix.encryptPacket(key, IV, content)
#        print time.clock()
        savetofile(newfilename, IV+ciphertext+mac)
    else:
        newfilename = filename[:-len(suffix)]
        okToProceed(newfilename)
        IV, ciphertext, mac = content[:16], content[16:-16], content[-16:]
        time.clock()
        plaintext, mac2 = pyPhelix.decryptPacket(key, IV, ciphertext)
#        print time.clock()
        if mac2 == mac:
            print '  *** good ***'
            savetofile(newfilename, plaintext)
        else:
            print '  *** bad ***'

#---------------------------------------------------------------

if __name__ == '__main__':
    main()
    
#---------------------------------------------------------------
#---------------------------------------------------------------
#---------------------------------------------------------------
